import au.com.bytecode.opencsv.CSVReader;

import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.*;

/**
 * Created by:
 * User: pjbitterman
 * Date: 3/1/13
 * Time: 12:33 PM
 */
public class Controller
{

    public static String inputPath = "../data/input/";
    public static String outputPath = "../data/output/";

    //FILE NAMES
    public static String farmCSV = "parcel_input.csv";  //land parcel table
    public static String scenarioCSV = "scenario_input";  //table of scenarios - drives the simulation
    public static String luTypesCSV = "land_use_types.csv";  //table of land uses, their code, costs, and crop type factors

    //PARAMETERS
    public static int randomSeed = 123;

    //OBJECTS and DATA STRUCTURES
    static Random randomGenerator = new Random(randomSeed);
    static ArrayList<Farmer> farmerList = new ArrayList<Farmer>();  //list of farmers
    static ArrayList<LandParcel> parcelList = new ArrayList<LandParcel>();  //list of landParcels
    static Perturbation thePerturbation = new Perturbation();  //perturbation that acts on the system in each scenario
    static ArrayList<String[]> parcelInputTable = new ArrayList<String[]>();
    static ArrayList<String[]> scenarioInputTable = new ArrayList<String[]>();
    static ArrayList<String[]> luInputFile = new ArrayList<String[]>();
    public static ArrayList<Scenario> scenarioArrayList = new ArrayList<Scenario>();

    /**
     * The main method runs the program
     * @param args arguments that aren't used in this program
     */
    public static void main(String[] args)
    {
        readScenarioFile();
        readParcelFile();
        readLUfile();
        int numScenarios = scenarioArrayList.size();  //how many scenarios there are in the simulation
        int currentScenario = 0; //the scenario currently running - start at 0

        //loops through the scenarios, running each in order
        //outputs after each run and then at the end

        long startTime = System.currentTimeMillis();
        while (currentScenario < numScenarios)
        {
            System.out.println("Now running scenario: " + currentScenario);


            Scenario relevantScenario = scenarioArrayList.get(currentScenario);
            runScenario(relevantScenario);  //runs each scenario a given amount of times (as read from file)
            resetScenario();  //reset objects for next scenario

            long endTime= System.currentTimeMillis();
            System.out.println("Scenario " + currentScenario + " took " + (endTime - startTime)/1000.0 + " seconds." );
            startTime = endTime;

            currentScenario++;  //increment to next scenario
        }
        outputSimulationResults();  //output for whole simulation todo: get from each simulation and output
    }


    /**
     * reads the scenario file
     */
    public static void readScenarioFile()
    {
        try
        {
            CSVReader tempFileReader = new CSVReader(new FileReader(inputPath + scenarioCSV));
            scenarioInputTable = new ArrayList<String[]>(tempFileReader.readAll());

            buildScenarioArrayList(scenarioInputTable);
        }
        catch (FileNotFoundException e)
        {
            e.printStackTrace();
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
    }

    public static void readParcelFile()
    {
        try
        {
            CSVReader tempFileReader = new CSVReader(new FileReader(inputPath + farmCSV));
            parcelInputTable = new ArrayList<String[]>(tempFileReader.readAll());
        }

        catch (FileNotFoundException e)
        {
            e.printStackTrace();
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
    }

    public static void readLUfile()
    {
        //setup static objects in LPClass
        try
        {
            CSVReader tempFileReader = new CSVReader(new FileReader(inputPath + luTypesCSV));
            luInputFile = new ArrayList<String[]>(tempFileReader.readAll());

        }
        catch (FileNotFoundException e)
        {
            e.printStackTrace();
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
    }

    public static void buildScenarioArrayList(ArrayList<String[]> inList)
    {
        for (int i = 1; i < inList.size(); i++) //start at 1 to avoid headers in location 0;
        {
            String[] toRead = inList.get(i);
            int scenarioID = Integer.parseInt(toRead[0]);
            int scenarioRuns = Integer.parseInt(toRead[1]);
            int runIterations = Integer.parseInt(toRead[2]);
            double cStartPrice = Double.parseDouble(toRead[3]);
            double bStartPrice = Double.parseDouble(toRead[4]);
            double sStartPrice = Double.parseDouble(toRead[5]);
            double crpStartPrice = Double.parseDouble(toRead[6]);
            double insurancePriceFactor = Double.parseDouble(toRead[7]);
            double temperature = Double.parseDouble(toRead[8]);
            double rFactor = Double.parseDouble(toRead[9]);  //used in usle
            double rainFall = Double.parseDouble(toRead[10]); //used in run-off

            boolean isDroughtPerturb = Boolean.parseBoolean(toRead[11]);
            int droughtDuration = Integer.parseInt(toRead[12]);
            double droughtMeanSeverity = Double.parseDouble(toRead[13]);

            boolean isPricePerturb = Boolean.parseBoolean(toRead[14]);
            int pricePerturbDuration = Integer.parseInt(toRead[15]);
            double cPricePerturbSeverity = Double.parseDouble(toRead[16]);
            double bPricePerturbSeverity = Double.parseDouble(toRead[17]);
            double sPricePerturbSeverity = Double.parseDouble(toRead[18]);

            int perturbationDelay = Integer.parseInt(toRead[19]);  //delay before perturbs start
            double switchSubsidy = Double.parseDouble(toRead[20]); //how much of switchgrass startup costs are subsidized
            double insuranceSubsidy = Double.parseDouble(toRead[21]);  //how much of insurance cost is subsidized

            int farmerTimeHorizon = Integer.parseInt(toRead[22]);
            int crpLeaseLength = Integer.parseInt(toRead[23]);

            Scenario toPut = new Scenario(scenarioID, scenarioRuns, runIterations, cStartPrice, bStartPrice,
                    sStartPrice, crpStartPrice, insurancePriceFactor, temperature, rFactor, rainFall, isDroughtPerturb,
                    droughtDuration, droughtMeanSeverity, isPricePerturb, pricePerturbDuration, cPricePerturbSeverity,
                    bPricePerturbSeverity, sPricePerturbSeverity, perturbationDelay, switchSubsidy, insuranceSubsidy,
                    farmerTimeHorizon, crpLeaseLength);

            scenarioArrayList.add(scenarioID, toPut);
        }
    }


    /**
     * Runs the individual scenario a given number of times
     * @param theScenario the scenario object to be run
     */
    public static void runScenario(Scenario theScenario)
    {
        setupSystem(theScenario);  //setup the system
        setupPerturbation(theScenario);  //setup the perturbation

        int currRun = 0;
        int numRuns = theScenario.getScenarioRuns();

        while (currRun < numRuns)  //repeat this scenario for given number of times
        {
            DataOrganizer myDataOrg = new DataOrganizer(outputPath);  //will hold the outputs at each tick and at end of each scenario run
            int currIteration = 0;
            int numIterations = theScenario.getRunIterations();
            randomSeed += 5;
            while (currIteration < numIterations)  //each run of the scenario has a number of "ticks" (years)
            {
                PolicyMaker.advanceListsToNextYear();
                decide();  //farmers decide land use
                buyInsurance();  //farmers decide whether to buy insurance
                plant();  //farmers plant crops
                grow();  //crops grow
                //perturb if it's a drought
                if (theScenario.getThePerturbation() != null)  //if there is a perturbation
                {
                    if(theScenario.getThePerturbation().isNaturePerturb()) doNaturePerturbation(currIteration, theScenario);
                }

                harvest();  //harvest whatever is in the field
                loseSoil();  //land parcels lose soil

                //perturb if it's a price perturbation (NOT a nature perturb)
                if(theScenario.getThePerturbation() != null)  //if there is a perturbation
                {
                    if (!theScenario.getThePerturbation().isNaturePerturb()) doPricePerturbation(currIteration, theScenario);
                }

                sell();  //farmers sell crops
                collectInsurance();  //collect if there was damage
                balanceBooks();  //farmers balance their books and collect any insurance

                //collect the data for each tick
                myDataOrg.orgTick(theScenario, currRun, currIteration, farmerList, parcelList);
                updateClimate();
                currIteration++;  //move to next year
            }
            myDataOrg.orgScenario(theScenario, farmerList, parcelList);
            myDataOrg.writeTicks(theScenario.getScenarioID(), currRun);
            resetSystemForNextRun(theScenario);

            currRun++;
        }

        writeScenarioSummary(theScenario);
    }

    /**
     * sets up the system based on the scenario of interest
     * @param currScenario the current scenario with parameters we care about
     */
    public static void setupSystem(Scenario currScenario)
    {
        setupLUStatics();
        setupParcels(parcelInputTable);
        setupFarmers(parcelInputTable, currScenario);
        setupNature(currScenario);
        setupMarket(currScenario);
        setupPolicyMaker(currScenario);

    }

    /**
     * sets up the perturbation based on the TYPE of perturbation
     * @param currScenario the scenario being run
     */
    private static void setupPerturbation(Scenario currScenario)
    {
        if (currScenario.isDroughtPerturb()) //is drought perturbation
        {
            assert(!currScenario.isPricePerturb());  //double check to make sure only one kind of perturbation

            Perturbation theDrought = new Perturbation();
            theDrought.setIsNaturePerturb(true);
            theDrought.setDuration(currScenario.getDroughtDuration());
            theDrought.setSeverity(currScenario.getDroughtMeanSeverity());
            currScenario.setThePerturbation(theDrought);
        }
        else if(currScenario.isPricePerturb())//is price perturbation
        {
            Perturbation thePricePerturbation = new Perturbation();
            thePricePerturbation.setIsNaturePerturb(false);
            currScenario.setThePerturbation((thePricePerturbation));
        }
        else //no perturbation
        {

        }
    }


    /**
     * Creates landParcels and puts them in the parcelList
     * @param inputTable the table of land parcels and who farms them
     */
    private static void setupParcels(ArrayList<String[]> inputTable)
    {

        for(int i = 1; i < inputTable.size(); i++)  //start at index 1 to avoid header
        {
            String[] tempStrParcel = inputTable.get(i);

            int tempParcelID;
            tempParcelID = i;
            int tempFarmerID = Integer.parseInt(tempStrParcel[0]);
            double tempAcre = Double.parseDouble(tempStrParcel[1]);
            double tempCSR = Double.parseDouble(tempStrParcel[2]);
            double tempHEL = Double.parseDouble(tempStrParcel[3]);
            int tempLastYrUse = Integer.parseInt((tempStrParcel[15]));
            int tempTypUse = Integer.parseInt(tempStrParcel[16]);
            double tempCornYield = Double.parseDouble(tempStrParcel[17]);
            double tempBeanYield = Double.parseDouble(tempStrParcel[18]);
            double tempWheatYield = Double.parseDouble(tempStrParcel[19]);
            double tempKfactor = Double.parseDouble(tempStrParcel[20]);
            double tempLSfactor = Double.parseDouble(tempStrParcel[21]);
            double tempTfactor = Double.parseDouble(tempStrParcel[22]);
            double tempXloc = Double.parseDouble(tempStrParcel[23]);
            double tempYloc = Double.parseDouble(tempStrParcel[24]);

            LandParcel toPut = new LandParcel();
            toPut.setParcelID(tempParcelID);
            toPut.setFarmerID(tempFarmerID);
            toPut.setAcres(tempAcre);
            toPut.setCornSuitabilityRating(tempCSR);
            toPut.setHel(tempHEL);
            toPut.updateLU(tempLastYrUse); //put this in first so that it's in index 0
            toPut.setTypicalUse(tempTypUse);
            toPut.updateLU(tempTypUse);  //put the first use to be the average of the last few years
            if (tempTypUse == 0)
            {
                //set years in crp to random between 1 and 10 for land initially in CRP
                toPut.setYrsInCrp(randomGenerator.nextInt(10)+1);
            }

            toPut.setPotentialCornYield(tempCornYield);
            toPut.setPotentialBeanYield(tempBeanYield);
            toPut.setPotentialWheatYield(tempWheatYield);
            toPut.setUsleK(tempKfactor);
            toPut.setUsleLS(tempLSfactor);
            toPut.setT_factor(tempTfactor);
            toPut.setxLoc(tempXloc);
            toPut.setyLoc(tempYloc);

            parcelList.add(toPut);
        }
    }

    /**
     * Creates farmers and puts them in the farmerList
     * @param inputTable the table of land parcels and who farms them
     */
    private static void setupFarmers(ArrayList<String[]> inputTable, Scenario currScenario)
    {
        //there are duplicate farmerIDs, so deal with them first
        ArrayList<Integer> listOfFarmerIDs = new ArrayList<Integer>();
        for(int i = 1; i < inputTable.size(); i++)  //start at index 1 to avoid header
        {
            String[] tempStrParcel = inputTable.get(i);
            int tempFarmerID = Integer.parseInt(tempStrParcel[0]);
            listOfFarmerIDs.add(tempFarmerID);
        }
        HashSet<Integer> hs = new HashSet<Integer>();   //hash gets rid of duplicate farmerIDs
        hs.addAll(listOfFarmerIDs);

        listOfFarmerIDs.clear();  //put back in the list
        listOfFarmerIDs.addAll(hs);
        Collections.sort(listOfFarmerIDs);   //sort

        //create farmers and put them in the list
        //keep in mind that not all farmerIDs are used - there can be gaps
        for(int i = 0; i < listOfFarmerIDs.size(); i++)
        {
            Farmer toPut = new Farmer();
            toPut.setFarmerID(listOfFarmerIDs.get(i));
            //add their time horizons
            toPut.setTimeHorizon(currScenario.getFarmerTimeHorizon());
            farmerList.add(i, toPut);
        }

        //go through the list of parcels, find the farmer they belong to,
        //and then add those parcels to that farmer's list of parcels they manage
        for (LandParcel toPut : parcelList)
        {
            int relevantFarmerID = toPut.getFarmerID();  //got farmerID

            //search through farmerList, find that farmer
            //addendum to the above

            for (Farmer testFarmer : farmerList)
            {
                if (testFarmer.getFarmerID() == relevantFarmerID)
                {
                    testFarmer.addToParcelList(toPut);
                }
            }
        }
    }

    /**
     * Sets up the nature class
     * @param currScenario the scenario that has the nature data
     */
    public static void setupNature(Scenario currScenario)
    {
        Nature.setPrecipitation(currScenario.getPrecipitation());
        Nature.setTemperature(currScenario.getTemperature());
        Nature.setStartingPrecip(currScenario.getPrecipitation());
        Nature.setTemperature(currScenario.getTemperature());
    }

    /**
     * Sets up the Market class - farms out a lot of the work to other methods
     * @param currScenario the scenario that has the market data
     */
    public static void setupMarket(Scenario currScenario)
    {
        setupMarketPrices(currScenario);
        setupSwitchGrassCosts();
        Market.setInsurancePriceFactor(currScenario.getInsurancePriceFactor());
    }

    /**
     * Sets up the prices for the Market class
     * @param currScenario the scenario that has the price data
     */
    public static void setupMarketPrices(Scenario currScenario)
    {
        Market.addLUPrice(0, currScenario.getCrpStartPrice());
        Market.addLUPrice(1, currScenario.getcStartPrice());
        Market.addLUPrice(2, currScenario.getbStartPrice());
        Market.addLUPrice(3, currScenario.getsStartPrice());
        Market.addLUPrice(4, 0.0); //out of production means no price

        Market.setInitialPrice(0, currScenario.getCrpStartPrice());
        Market.setInitialPrice(1, currScenario.getcStartPrice()) ;
        Market.setInitialPrice(2, currScenario.getbStartPrice());
        Market.setInitialPrice(3, currScenario.getsStartPrice());
        Market.setInitialPrice(4, 0.0);  //out of production means no sale price

        Market.setInsuranceReimbursementFactor(currScenario.getInsuranceReimbursementFactor());
        Market.setInsurancePriceFactor(currScenario.getInsurancePriceFactor());
    }

    /**
     * Sets up switchgrass costs
     * done separately from other land uses since their costs depend on continuous years
     */
    public static void setupSwitchGrassCosts()
    {
        ArrayList<Double> sgCostList = new ArrayList<Double>(Arrays.asList(175.0, 113.66, 105.0, 97.0, 89.0, 80.0));
        //todo: eventually move the above costs to an input file
        Market.setSwitchGrassCostList(sgCostList);
    }

    /**
     * sets up the PolicyMaker class
     * @param currScenario the scenario that has the subsidy and other policy data
     */
    public static void setupPolicyMaker(Scenario currScenario)
    {
        PolicyMaker.setSwitchGrassSubsidy(currScenario.getSwitchGrassSubsidy());
        PolicyMaker.setInsurancePriceFactor(currScenario.getInsurancePriceFactor());
        PolicyMaker.setCrpLeaseLength(currScenario.getCrpLeaseLength());
    }

    /**
     * sets up miscellaneous static variables, primarily the land use file
     */
    public static void setupLUStatics()
    {
        //setup static objects in LPClass

        String[] luNames = luInputFile.get(0);  //the names are in first row
        for (int i = 1; i < luNames.length; i++)  //start at index 1, since there's a descriptive first column as well
        {
            LandParcel.setLUClass(i-1, luNames[i]);
        }

        String[] luCodes = luInputFile.get(1); //codes in second row
        for (int i = 1; i < luCodes.length; i++)
        {
            //the codes are the indices as well, so no real need to do anything with this
        }

        String[] luCosts = luInputFile.get(2); //mean costs in third row
        for (int i = 1; i < luCosts.length; i++)
        {
            int luCode = Integer.parseInt(luCodes[i]);
            double luCost = Double.parseDouble(luCosts[i]);
            Market.addLUCost(luCode, luCost);
        }

        String[] luCTFactors = luInputFile.get(3); //ct factors in fourth row
        for (int i = 1; i < luCTFactors.length; i++)
        {
            int luCode = Integer.parseInt(luCodes[i]);
            double luCT = Double.parseDouble(luCTFactors[i]);
            LandParcel.addUsleCT(luCode, luCT);
        }

    }

    /**
     * tells farmers to decide their land use for the following year
     */
    public static void decide()
    {
        //shuffle the list so decisions are made in a different order every iteration
        ArrayList<Farmer> shuffledList = new ArrayList<Farmer>(farmerList);
        Collections.shuffle(shuffledList, randomGenerator);

        for (Farmer relevantFarmer : shuffledList)
        {
            relevantFarmer.decideLU();
        }
    }

    public static void buyInsurance()
    {
        for (Farmer f: farmerList) f.buyInsurance();
    }


    /**
     * farmers plant their crops
     */
    public static void plant()
    {
        for (Farmer f : farmerList)
        {
            f.plantCrops();
        }
    }

    /**
     * crops grow on individual landParcels
     */
    public static void grow()
    {
        for (LandParcel relevantParcel : parcelList)
        {
            relevantParcel.growCrops();
        }
    }

    /**
     * Perturbs the system via Nature
     * @param currIteration the "year" the system is currently on
     */
    public static void doNaturePerturbation(int currIteration, Scenario currScenario)
    {
        Perturbation thePerturbation = currScenario.getThePerturbation();

        //if perturb hasn't started and is after the delay
        if(!thePerturbation.isHasStarted() && currIteration > currScenario.getPerturbationDelay())
        {
            int random_factor = Controller.randomGenerator.nextInt(8); //12.5% chance of starting
            if (random_factor == 0)  //test to see if it matches the random
            {
                currScenario.setPerturbStartTick(currIteration);
                thePerturbation.setHasStarted(true);
                Nature.setPrecipitation(Nature.getPrecipitation() * thePerturbation.getSeverity());
                Nature.setTemperature(Nature.getTemperature() * 1.07);
                Nature.setUsleR(Nature.getUsleR()*.50);
                for (LandParcel lp : parcelList)
                {
                    if(lp.getCurrLU() != 0) //if NOT CRP
                    {
                        double currCrops = lp.getCropInField();
                        double severity = thePerturbation.getSeverity();

                        //the perrenial isn't affected as much
                        //todo: parameterize eventually
                        if(lp.getCurrLU() == 3)
                        {
                            severity = .54 * severity + .49;  //just a basic linear relationship - again, parameterize
                            if (severity > 1) severity = 1;
                        }

                        double factor = 1 + randomGenerator.nextGaussian()/10;  //minor variation
                        lp.setCropInField(currCrops * severity * factor);
                    }
                }
                thePerturbation.setTicksApplied(thePerturbation.getTicksApplied() + 1);
                thePerturbation.updateDoneness();
            }
        }

        //if the perturbation has started by not finished
        if(thePerturbation.isHasStarted() && !thePerturbation.isHasFinished())
        {
            for (LandParcel lp : parcelList)
            {
                if(lp.getCurrLU() != 0) //if NOT CRP
                {
                    double currCrops = lp.getCropInField();
                    double severity = thePerturbation.getSeverity();
                    double factor = 1 + randomGenerator.nextGaussian()/20;  //minor variation
                    lp.setCropInField(currCrops * severity * factor);
                }
            }
            thePerturbation.setTicksApplied(thePerturbation.getTicksApplied() + 1);
            thePerturbation.updateDoneness();
        }

        //if the nature perturb is finished, reset precipitation and temp
        if(thePerturbation.isHasFinished())
        {
            Nature.setPrecipitation(Nature.getStartingPrecip());
            Nature.setTemperature(Nature.getStartingTemp());
            Nature.setUsleR(Nature.getStartingUsleR());
        }

    }

    /**
     * the farmers harvest their crops from their landParcels
     */
    public static void harvest()
    {
        for (Farmer f: farmerList)
        {
            f.harvestCrops();
        }
    }

    /**
     * the landParcels lose soil based on what was cropped
     */
    public static void loseSoil()
    {
        for(LandParcel lp: parcelList)
        {
            lp.loseSoil();
        }
    }

    /**
     * Perturbs the system via Markets
     * @param currIteration the "year" the system is on
     */
    public static void doPricePerturbation(int currIteration, Scenario currScenario)
    {
        Perturbation thePerturbation = currScenario.getThePerturbation();

        //if perturb hasn't started and is after the delay
        if(!thePerturbation.isHasStarted() && currIteration > currScenario.getPerturbationDelay())
        {
            int random_factor = Controller.randomGenerator.nextInt(10); //10% chance of starting at any given tick
            if (random_factor == 0)  //test to see if it matches the random
            {
                currScenario.setPerturbStartTick(currIteration);
                thePerturbation.setHasStarted(true);
                double currPriceCorn = Market.getLUPrice(1);
                double currPriceBeans = Market.getLUPrice(2);
                double currPriceSwitch = Market.getLUPrice(3);
                double cornSeverity = currScenario.getcPricePerturbSeverity();
                double beanSeverity = currScenario.getbPricePerturbSeverity();
                double switchSeverity = currScenario.getsPricePerturbSeverity();
                double factor = 1 + randomGenerator.nextGaussian()/20;  //minor variation

                Market.updateLUPrice(1, factor * cornSeverity * currPriceCorn);
                Market.updateLUPrice(2, factor * beanSeverity * currPriceBeans);
                Market.updateLUPrice(3, factor * switchSeverity * currPriceSwitch);

                thePerturbation.setTicksApplied(thePerturbation.getTicksApplied() + 1);
                thePerturbation.updateDoneness();
            }
        }

        //if the perturbation has started by not finished, don't update prices because they should still be low
        if(thePerturbation.isHasStarted() && !thePerturbation.isHasFinished())
        {
            thePerturbation.setTicksApplied(thePerturbation.getTicksApplied() + 1);
            thePerturbation.updateDoneness();
        }

        //if the perturb is finished, reset prices back towards original prices
        if(thePerturbation.isHasFinished())
        {
            for(int i = 0; i < LandParcel.getNumberLUClasses(); i++)  //cycle through land use classes, reset prices if there isn't any perturbation
            {
                Market.updateLUPrice(i, Market.getInitialPrice(i));
            }
        }
    }

    /**
     * farmers sell their harvested crops
     */
    public static void sell()
    {
        for (Farmer f: farmerList)
        {
            f.sellCrops();
        }
    }

    /**
     * farmers collect any insurance payments if necessary
     */
    public static void collectInsurance()
    {

        for (Farmer f : farmerList)
        {

            f.collectInsurancePayment();
        }
    }

    /**
     * farmers balance their books and check to make sure everything went well
     */
    public static void balanceBooks()
    {
        for (Farmer f : farmerList)
        {
            f.balanceBooks();
        }
    }

    public static void updateClimate()
    {
        Nature.setTemperature(Nature.getTemperature() * 1.001);  //todo: a magic number at the moment, but is ~5% gain in 50 yrs
    }

    public static void writeScenarioSummary(Scenario theScenario)
    {
        MyFileWriter fw = new MyFileWriter();
        fw.writeScenarioSummary(theScenario);
    }


    /**
     * outputs the results of all scenarios
     */
    public static void outputSimulationResults()
    {
        //todo: do this later - manually at the moment
    }

    public static void resetSystemForNextRun(Scenario currScenario)
    {
        setupLUStatics();

        parcelList = new ArrayList<LandParcel>();
        farmerList = new ArrayList<Farmer>();


        setupMarket(currScenario);

        PolicyMaker.resetPolicyMaker();

        setupParcels(parcelInputTable);
        setupFarmers(parcelInputTable, currScenario);

        setupPerturbation(currScenario);
        setupNature(currScenario);
        setupPolicyMaker(currScenario);
    }

    /**
     * resets the scenario after the completion of each run
     */
    public static void resetScenario()
    {
        //new lists
        parcelList = new ArrayList<LandParcel>();
        farmerList = new ArrayList<Farmer>();

        //reset statics for various classes
        LandParcel.resetLandParcel();
        Market.resetMarket();
        PolicyMaker.resetPolicyMaker();
        //todo: figure out Nature reset - probably have to move variables to the scenario file

        //new objects
        thePerturbation = new Perturbation();
    }
}
