
import au.com.bytecode.opencsv.CSVWriter;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

/**
 * Created with IntelliJ IDEA.
 * User: pjbitterman
 * Date: 3/1/13
 * Time: 12:37 PM
 */

public class DataOrganizer
{
    private String outputPath;
    private ArrayList<Integer> tick_scenarioID = new ArrayList<Integer>();
    private ArrayList<Integer> tick_iteration = new ArrayList<Integer>();
    private ArrayList<Integer> tick_run_number = new ArrayList<Integer>();
    private ArrayList<Integer> tick_count_farmers = new ArrayList<Integer>();
    private ArrayList<Integer> tick_farmers_in_prod = new ArrayList<Integer>();
    private ArrayList<Boolean> tick_has_started = new ArrayList<Boolean>();
    private ArrayList<Boolean> tick_has_finished = new ArrayList<Boolean>();
    private ArrayList<Double>  tick_corn_yield = new ArrayList<Double>();
    private ArrayList<Double>  tick_beans_yield = new ArrayList<Double>();
    private ArrayList<Double>  tick_switch_yield = new ArrayList<Double>();
    private ArrayList<Double>  tick_sus_metric = new ArrayList<Double>();
    private ArrayList<Double>  tick_net_sum = new ArrayList<Double>();
    private ArrayList<Double>  tick_net_mean = new ArrayList<Double>();
    private ArrayList<Double>  tick_net_sd = new ArrayList<Double>();
    private ArrayList<Double>  tick_gross_sum = new ArrayList<Double>();
    private ArrayList<Double>  tick_gross_mean = new ArrayList<Double>();
    private ArrayList<Double>  tick_gross_sd = new ArrayList<Double>();
    private ArrayList<Double>  tick_costs_sum = new ArrayList<Double>();
    private ArrayList<Double>  tick_costs_mean = new ArrayList<Double>();
    private ArrayList<Double>  tick_costs_sd = new ArrayList<Double>();
    private ArrayList<Integer> tick_num_parcels = new ArrayList<Integer>();
    private ArrayList<Integer> tick_num_parcels_in_prod = new ArrayList<Integer>();
    private ArrayList<Integer> tick_num_parcels_crp = new ArrayList<Integer>();
    private ArrayList<Integer> tick_num_parcels_corn = new ArrayList<Integer>();
    private ArrayList<Integer> tick_num_parcels_beans = new ArrayList<Integer>();
    private ArrayList<Integer> tick_num_parcels_switch = new ArrayList<Integer>();
    private ArrayList<Integer> tick_num_parcels_oop = new ArrayList<Integer>();
    private ArrayList<Double>  tick_parcel_entropy = new ArrayList<Double>();
    private ArrayList<Double>  tick_num_acres_crp = new ArrayList<Double>();
    private ArrayList<Double>  tick_num_acres_corn = new ArrayList<Double>();
    private ArrayList<Double>  tick_num_acres_beans = new ArrayList<Double>();
    private ArrayList<Double>  tick_num_acres_switch = new ArrayList<Double>();
    private ArrayList<Double>  tick_num_acres_oop = new ArrayList<Double>();
    private ArrayList<Double>  tick_run_off = new ArrayList<Double>();
    private ArrayList<Double>  tick_soil_lost = new ArrayList<Double>();
    private ArrayList<Double>  tick_ins_payment = new ArrayList<Double>();
    private ArrayList<Double>  tick_switch_subsidy = new ArrayList<Double>();
    private ArrayList<Double>  tick_avg_farm_parcels = new ArrayList<Double>();
    private ArrayList<Double>  tick_avg_farm_acres = new ArrayList<Double>();


    public DataOrganizer(String outputPath)
    {
        this.outputPath = outputPath;
    }



    public void orgTick(Scenario currScenario, int currRun, int currIteration, ArrayList<Farmer> farmerList, ArrayList<LandParcel> parcelList)
    {
        int sID = currScenario.getScenarioID();
        tick_scenarioID.add(sID);

        tick_run_number.add(currRun);

        tick_iteration.add(currIteration);

        int numFarmers = farmerList.size();
        tick_count_farmers.add(numFarmers);

        int farmersInProduction = 0;
        for(Farmer f: farmerList)
        {
            if (f.isInBusiness()) farmersInProduction++;
        }
        tick_farmers_in_prod.add(farmersInProduction);

        boolean hasStarted = currScenario.getThePerturbation().isHasStarted();
        tick_has_started.add(hasStarted);

        boolean hasFinished = currScenario.getThePerturbation().isHasFinished();
        tick_has_finished.add(hasFinished);

        //parcel stuff
        double tempCyield = 0;
        double tempByield = 0;
        double tempSyield = 0;
        int[] luCounts = new int[LandParcel.getNumberLUClasses()];
        double[] luAcres = new double[LandParcel.getNumberLUClasses()];
        int tempParcelsInProd = 0;
        double tempAcresInProd = 0;

        for (LandParcel lp: parcelList)
        {
            double tempYield = lp.getYield();
            int lu = lp.getCurrLU();
            luCounts[lu] = luCounts[lu] + 1;
            luAcres[lu] = luAcres[lu] + lp.getAcres();

            if(lp.isInProduction())
            {
                tempParcelsInProd++;
                tempAcresInProd += lp.getAcres();
            }
            if(lu == 1) tempCyield += tempYield;
            if(lu == 2) tempByield += tempYield;
            if(lu == 3) tempSyield += tempYield;
        }
        tick_num_parcels.add(parcelList.size());
        tick_num_parcels_in_prod.add(tempParcelsInProd);
        tick_num_parcels_crp.add(luCounts[0]);
        tick_num_parcels_corn.add(luCounts[1]);
        tick_num_parcels_beans.add(luCounts[2]);
        tick_num_parcels_switch.add(luCounts[3]);
        tick_num_parcels_oop.add(luCounts[4]);

 
        //modified simpson diversity index
        double running = 0;
        for (int lu: luCounts)
        {
            double p_i = (double) lu / parcelList.size();
            double p_i_sqd = Math.pow(p_i, 2);
            running += p_i_sqd;
        }
        double simpson = 1 - running;
        double modSimpson = Math.log(running) * -1;
        tick_parcel_entropy.add(modSimpson);



        tick_avg_farm_parcels.add((double)tempParcelsInProd / farmersInProduction);
        tick_avg_farm_acres.add(tempAcresInProd / farmersInProduction);

        //acres and yield
        tick_num_acres_crp.add(luAcres[0]);
        tick_num_acres_corn.add(luAcres[1]);
        tick_num_acres_beans.add(luAcres[2]);
        tick_num_acres_switch.add(luAcres[3]);
        tick_num_acres_oop.add(luAcres[4]);
        tick_corn_yield.add(tempCyield);
        tick_beans_yield.add(tempByield);
        tick_switch_yield.add(tempSyield);

        //farmer stuff
        double tempNetSum = 0;
        double tempGrossSum = 0;
        double tempCostsSum = 0;

        for(Farmer f: farmerList)
        {
            tempNetSum += f.getCurrentYrNet();
            tempCostsSum += f.getCurrentYrCosts();
            tempGrossSum += f.getCurrentYrGross();
        }

        double tempNetMean = tempNetSum / farmerList.size();
        double tempGrossMean = tempGrossSum / farmerList.size();
        double tempCostsMean = tempCostsSum / farmerList.size();

        double devSum_net = 0;
        double devSum_gross = 0;
        double devSum_costs = 0;
        for(Farmer f: farmerList)
        {
            double net = f.getCurrentYrNet();
            double costs = f.getCurrentYrCosts();
            double gross = f.getCurrentYrGross();

            devSum_net += Math.pow(net - tempNetMean, 2);
            devSum_gross += Math.pow(gross - tempGrossMean, 2);
            devSum_costs += Math.pow(costs - tempCostsMean, 2);
        }

        double tempNetSD = Math.sqrt(devSum_net/farmerList.size());
        double tempGrossSD = Math.sqrt(devSum_gross/farmerList.size());
        double tempCostsSD = Math.sqrt(devSum_costs/farmerList.size());

        tick_net_sum.add(tempNetSum);
        tick_net_mean.add(tempNetMean);
        tick_net_sd.add(tempNetSD);
        tick_gross_sum.add(tempGrossSum);
        tick_gross_mean.add(tempGrossMean);
        tick_gross_sd.add(tempGrossSD);
        tick_costs_sum.add(tempCostsSum);
        tick_costs_mean.add(tempCostsMean);
        tick_costs_sd.add(tempCostsSD);

        double tempRunOff = 0;
        double tempSoilLost = 0;
        for(LandParcel lp: parcelList)
        {
            tempRunOff += lp.calcRunoff();
            tempSoilLost += lp.calcUSLE();
        }
        tick_run_off.add(tempRunOff);
        tick_soil_lost.add(tempSoilLost);

        tick_ins_payment.add(PolicyMaker.getCurrYrInsuranceTotal());
        tick_switch_subsidy.add(PolicyMaker.getCurrYrSubsidyTotal());

    }

    public void orgScenario(Scenario currScenario, ArrayList<Farmer> farmerList, ArrayList<LandParcel> parcelList)
    {
        int tempSID = currScenario.getScenarioID();
        int tempTicks = this.tick_run_number.size();
        int tempFarmerHorizon = currScenario.getFarmerTimeHorizon();

        double tempSumNet = 0;
        double tempSumCosts = 0;
        double tempSumGross = 0;
        for (int i = 0; i < tick_net_sum.size(); i++)
        {
            tempSumNet += tick_net_sum.get(i);
            tempSumCosts += tick_costs_sum.get(i);
            tempSumGross += tick_gross_sum.get(i);
        }

        double tempRunOff = 0;
        for (Double d: this.tick_run_off)
        {
            tempRunOff += d;
        }

        double tempSoilLost = 0;
        for (Double d: this.tick_soil_lost)
        {
            tempSoilLost += d;
        }

        double insPayment = 0;
        for (Double d: this.tick_ins_payment)
        {
            insPayment += d;
        }

        double subPayments = 0;
        for (Double d: this.tick_switch_subsidy)
        {
            subPayments += d;
        }

        double totCornYield = 0;
        for (Double d: this.tick_corn_yield)
        {
            totCornYield += d;
        }

        double totBeanYield = 0;
        for (Double d: this.tick_beans_yield)
        {
            totBeanYield += d;
        }
        double totSwitchYield = 0;
        for (Double d: this.tick_switch_yield)
        {
            totSwitchYield += d;
        }

        int tempPerturbTick = 0;
        {
            tempPerturbTick = currScenario.getPerturbStartTick();
        }


        int tempFarmersInBusiness = 0;
        for(Farmer f: farmerList)
        {
            if (f.isInBusiness()) tempFarmersInBusiness++;
        }

        int tempLPinProduction = 0;
        double tempAcresinProduction = 0;

        int tempLPinSwitch = 0;
        double tempAcresInSwitch = 0;
        for(LandParcel lp: parcelList)
        {
            if(lp.isInProduction())
            {
                tempLPinProduction++;
                tempAcresinProduction += (lp.getAcres());
            }

            if(lp.getCurrLU() == 3)
            {
                tempLPinSwitch++;
                tempAcresInSwitch += (lp.getAcres());
            }
        }

        double tempEndFarmSizeLP = this.tick_avg_farm_parcels.get(tick_avg_farm_parcels.size() - 1);
        double tempEndFarmSizeAcres = this.tick_avg_farm_acres.get(tick_avg_farm_acres.size() - 1);
        double tempEndLandscapeEntropy = this.tick_parcel_entropy.get(tick_parcel_entropy.size() - 1);

        double tempStartFarmSizeLP = this.tick_avg_farm_parcels.get(3);  //tick 3 isn't the start, but allows for it to find equilibrium
        double tempStartFarmSizeAcres = this.tick_avg_farm_acres.get(3);
        double tempStartLandscapeEntropy = this.tick_parcel_entropy.get(3);

        currScenario.endOfRun(tempSID, tempTicks, tempFarmerHorizon, tempSumGross, tempSumCosts, tempSumNet, tempRunOff,
                tempSoilLost, insPayment, subPayments, totCornYield, totBeanYield, totSwitchYield, tempPerturbTick,
                tempFarmersInBusiness, tempLPinProduction, tempAcresinProduction, tempLPinSwitch, tempAcresInSwitch,
                tempStartFarmSizeLP, tempEndFarmSizeLP, tempStartFarmSizeAcres, tempEndFarmSizeAcres,
                tempStartLandscapeEntropy, tempEndLandscapeEntropy);

    }

    public void writeTicks(int scenarioID, int currRun)
    {
        ArrayList<String[]> toWrite = new ArrayList<String[]>();

        String[] headArray = {"scenario ID" , "iteration", "run number", "number of farmers","farmers in biz", "perturb has started",
                "perturb has finished", "corn yield","bean yield", "switch yield", "sus metric", "net sum", "net mean",
                "net sd", "gross sum", "gross mean", "gross sd","costs sum", "costs mean", "costs sd", "net mean of farmers remaining",
                "num parcels", "num parcels in prod", "num parcels crp", "num parcels corn", "num parcels beans", "num parcels switch",
                "num parcels oop", "num parcels row crops", "landscape entropy", "num acres crp", "num acres corn", "num acres beans","num acres switch",
                "num acres oop", "num acres row crops", "proportion parcels in prod", "avg farm size parcels", "avg farm size acres", "run-off", "soil lost", "ins payments", "switch sub"};

        toWrite.add(headArray);

        //have to wait for the end to do the sus metric:
        tick_sus_metric = calcSusMetric();


        for (int i = 0; i < tick_iteration.size(); i++)
        {
            String[] tickArray = new String[43];
            tickArray[0] = Integer.toString(tick_scenarioID.get(i));
            tickArray[1] = Integer.toString(tick_iteration.get(i));
            tickArray[2] = Integer.toString(tick_run_number.get(i));
            tickArray[3] = Integer.toString(tick_count_farmers.get(i));
            tickArray[4] = Integer.toString(tick_farmers_in_prod.get(i));
            tickArray[5] = Boolean.toString(tick_has_started.get(i));
            tickArray[6] =  Boolean.toString(tick_has_finished.get(i));
            tickArray[7] =  Double.toString(tick_corn_yield.get(i));
            tickArray[8] =  Double.toString(tick_beans_yield.get(i));
            tickArray[9] =  Double.toString(tick_switch_yield.get(i));
            tickArray[10] =  Double.toString(tick_sus_metric.get(i));
            tickArray[11] =  Double.toString(tick_net_sum.get(i));
            tickArray[12] =  Double.toString(tick_net_mean.get(i));
            tickArray[13] =  Double.toString(tick_net_sd.get(i));
            tickArray[14] =  Double.toString(tick_gross_sum.get(i));
            tickArray[15] =  Double.toString(tick_gross_mean.get(i));
            tickArray[16] =  Double.toString(tick_gross_sd.get(i));
            tickArray[17] =  Double.toString(tick_costs_sum.get(i));
            tickArray[18] =  Double.toString(tick_costs_mean.get(i));
            tickArray[19] =  Double.toString(tick_costs_sd.get(i));
            tickArray[20] =  Double.toString(tick_net_sum.get(i) / tick_farmers_in_prod.get(i));
            tickArray[21] =  Integer.toString(tick_num_parcels.get(i));
            tickArray[22] =  Integer.toString(tick_num_parcels_in_prod.get(i));
            tickArray[23] =  Integer.toString(tick_num_parcels_crp.get(i));
            tickArray[24] =  Integer.toString(tick_num_parcels_corn.get(i));
            tickArray[25] =  Integer.toString(tick_num_parcels_beans.get(i));
            tickArray[26] =  Integer.toString(tick_num_parcels_switch.get(i));
            tickArray[27] =  Integer.toString(tick_num_parcels_oop.get(i));
            tickArray[28] =  Integer.toString(tick_num_parcels_corn.get(i) + tick_num_parcels_beans.get(i));
            tickArray[29] =  Double.toString(tick_parcel_entropy.get(i));
            tickArray[30] =  Double.toString(tick_num_acres_crp.get(i));
            tickArray[31] =  Double.toString(tick_num_acres_corn.get(i));
            tickArray[32] =  Double.toString(tick_num_acres_beans.get(i));
            tickArray[33] =  Double.toString(tick_num_acres_switch.get(i));
            tickArray[34] =  Double.toString(tick_num_acres_oop.get(i));
            tickArray[35] =  Double.toString(tick_num_acres_corn.get(i) + tick_num_acres_beans.get(i));
            tickArray[36] =  Double.toString(tick_num_parcels_in_prod.get(i)/(double)tick_num_parcels.get(i));
            tickArray[37] =  Double.toString(tick_avg_farm_parcels.get(i));
            tickArray[38] =  Double.toString(tick_avg_farm_acres.get(i));
            tickArray[39] =  Double.toString(tick_run_off.get(i));
            tickArray[40] =  Double.toString(tick_soil_lost.get(i));
            tickArray[41] =  Double.toString(tick_ins_payment.get(i));
            tickArray[42] =  Double.toString(tick_switch_subsidy.get(i));

            toWrite.add(tickArray);
        }
        try
        {
            String sID = Integer.toString(scenarioID);
            String runNum = Integer.toString(currRun);

            String dirPath = outputPath + "/" + getDateTime() + "/" + "sc_" + sID + "/";
            File file = new File(dirPath);

            if (!file.exists()) file.mkdirs();

            FileWriter fWriter = new FileWriter(outputPath + "/many/" + sID + "_run" + runNum + ".csv");  //for direct writing
//            FileWriter fWriter = new FileWriter(dirPath + "sc" + sID + "_run" + runNum + "_" + getDateTime() + ".csv");
            CSVWriter myWriter = new CSVWriter(fWriter, ',');
            myWriter.writeAll(toWrite);
            myWriter.close();
        }
        catch (IOException e)
        {
            System.out.println("Unable to create output file " + e.getMessage());
        }
    }

    private String getDateTime() {
        DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        Date date = new Date();
        return dateFormat.format(date);
    }

    private  ArrayList<Double> calcSusMetric()
    {
        ArrayList<Double> toReturn = new ArrayList<Double>();

        //find max and mins
        double run_off_min = Double.MAX_VALUE;
        double run_off_max = Double.MIN_VALUE;
        double soil_loss_min = Double.MAX_VALUE;
        double soil_loss_max = Double.MIN_VALUE;
        double net_mean_min = Double.MAX_VALUE;
        double net_mean_max = Double.MIN_VALUE;
        double net_sd_min = Double.MAX_VALUE;
        double net_sd_max = Double.MIN_VALUE;
        double subsidy_min = Double.MAX_VALUE;
        double subsidy_max = Double.MIN_VALUE;
        double ins_pay_min = Double.MAX_VALUE;
        double ins_pay_max = Double.MIN_VALUE;

        for(double d: tick_run_off)
        {
            if (d > run_off_max) run_off_max = d;
            if (d < run_off_min) run_off_min = d;
        }

        for(double d: tick_soil_lost)
        {
            if (d > soil_loss_max) soil_loss_max = d;
            if (d < soil_loss_min) soil_loss_min = d;
        }

        for(double d: tick_net_mean)
        {
            if (d > net_mean_max) net_mean_max = d;
            if (d < net_mean_min) net_mean_min = d;
        }

        for(double d: tick_net_sd)
        {
            if (d > net_sd_max) net_sd_max = d;
            if (d < net_sd_min) net_sd_min = d;
        }

        for(double d: tick_switch_subsidy)
        {
            if (d > subsidy_max) subsidy_max = d;
            if (d < subsidy_min) subsidy_min = d;
        }

        for(double d: tick_ins_payment)
        {
            if (d > ins_pay_max) ins_pay_max = d;
            if (d < ins_pay_min) ins_pay_min = d;
        }

        for (int i = 0; i < tick_run_off.size(); i++)  //all lists should be same size
        {
            // -1's are to switch the sign based on desirability
            double ru = 1 -  (tick_run_off.get(i) - run_off_min) / (run_off_max - run_off_min);
            double sl = 1 - (tick_soil_lost.get(i) - soil_loss_min) / (soil_loss_max - soil_loss_min);
            double nm = (tick_net_mean.get(i) - net_mean_min) / (net_mean_max - net_mean_min);
            double nsd = 1 - (tick_net_sd.get(i) - net_sd_min) / (net_sd_max - net_sd_min);
            double ss = 1 - (tick_switch_subsidy.get(i) - subsidy_min) / (subsidy_max - subsidy_min);
            double ins = 1 - (tick_ins_payment.get(i) - ins_pay_min) / (ins_pay_max - ins_pay_min);


            double multiplied = ru * sl * nm * nsd * ss * ins;
            double num_terms = 6;

            double sus_index = Math.pow(multiplied, (1/num_terms));

            toReturn.add(sus_index);
        }

        return toReturn;
    }





}