/**
 * Created with IntelliJ IDEA.
 * User: pjbitterman
 * Date: 3/1/13
 * Time: 12:34 PM
 */


import java.util.ArrayList;



public class Farmer
{
    //instance variables
    private int farmerID;
    private ArrayList<LandParcel> indParcelList = new ArrayList<LandParcel>();
    private double[] annualProduction;  //how much of each crop the farmer harvests
    private int timeHorizon;  //how far into the future each farmer looks to make decisions

    //history of costs, sales, and net - most recent yr is always last index
    private ArrayList<Double> annualCosts = new ArrayList<Double>();
    private ArrayList<Double> annualGross = new ArrayList<Double>();
    private ArrayList<Double> annualInsurance = new ArrayList<Double>();

    private double expectedAnnualNet = 0;

    private boolean isInBusiness;

    private boolean isAdaptable;

    //constructor
    public Farmer()
    {
        //zero initial production
        annualProduction = new double[LandParcel.LUClasses.length];

        isAdaptable = true;

        for (int i = 0; i < annualProduction.length; i++)
        {
            annualProduction[i] = 0;
            this.setInBusiness(true);

        }
    }

    public int getFarmerID()
    {
        return farmerID;
    }

    public void setFarmerID(int farmerID)
    {
        this.farmerID = farmerID;
    }

    public boolean isInBusiness()
    {
        return isInBusiness;
    }

    public void setInBusiness(boolean inBusiness)
    {
        isInBusiness = inBusiness;
    }

    public ArrayList<LandParcel> getIndParcelList()
    {
        return indParcelList;
    }

    public void setIndParcelList(ArrayList<LandParcel> indParcelList)
    {
        this.indParcelList = indParcelList;
    }

    public void addToParcelList(LandParcel theParcel)
    {
        ArrayList<LandParcel> workingList = this.getIndParcelList();
        workingList.add(theParcel);
    }

    public double getAnnualCosts(int index)
    {
        return annualCosts.get(index);
    }

    public void addAnnualCosts(double annualCosts)
    {
        this.annualCosts.add(annualCosts);
    }

    public void setAnnualCosts(int index, double amount)
    {
        this.annualCosts.set(index, amount);
    }

    public double getAnnualGross(int index)
    {
        return annualGross.get(index);
    }

    public void addAnnualGross(double annualGross)
    {
        this.annualGross.add(annualGross);
    }

    public void setAnnualGross(int index, double amount)
    {
        this.annualGross.set(index, amount);
    }

    public double getAnnualNet(int index)
    {
        return annualGross.get(index) - annualCosts.get(index);
    }

    public int getTimeHorizon()
    {
        return timeHorizon;
    }

    public void setTimeHorizon(int timeHorizon)
    {
        this.timeHorizon = timeHorizon;
    }

    public boolean isAdaptable()
    {
        return isAdaptable;
    }

    public void setAdaptable(boolean isAdaptable)
    {
        this.isAdaptable = isAdaptable;
    }


    public double getCurrentYrCosts()
    {
        return this.getAnnualCosts(this.annualCosts.size() - 1);
    }

    public double getCurrentYrNet()
    {
        int currYr = this.annualGross.size() - 1;
        return this.getAnnualGross(currYr) - this.getAnnualCosts(currYr);
    }

    public double getCurrentYrGross()
    {
        int currYr = this.annualGross.size() - 1;
        return this.getAnnualGross(currYr);
    }

    public double getExpectedAnnualNet()
    {
        return expectedAnnualNet;
    }

    public void setExpectedAnnualNet(double expectedAnnualNet)
    {
        this.expectedAnnualNet = expectedAnnualNet;
    }

    public void addAnnualInsurance(double payment)
    {
        this.annualInsurance.add(payment);
    }

    public double getAnnualInsurance(int index)
    {
        return this.annualInsurance.get(index);
    }

    public double getAnnualProduction(int index)
    {
        return this.annualProduction[index];
    }

    public void setAnnualProduction(int lu, double amount)
    {
        this.annualProduction[lu] = amount;
    }


    public void decideLU()
    {
        for (LandParcel lp : this.indParcelList)
        {
            int luToPlant;
            double acres = lp.getAcres();
            double hel = lp.getHel();
            int numClasses = LandParcel.getNumberLUClasses();

            //the index values of this array correspond to lu classes
            ArrayList<Double> expectedNet = new ArrayList<Double>();
            ArrayList<Double> expectedCosts = new ArrayList<Double>();
            ArrayList<Double> expectedSales = new ArrayList<Double>();

            //put non-profitable plots out of production
            double lastNyearsSum = 10000;
            if(this.annualCosts.size() > 5) //wait a few years for this test
            {
                int listSize = this.annualCosts.size();
                int numYears = 3;
                int backCounter = listSize - 1;
                int breakOff = listSize - numYears;
                while (backCounter > breakOff)
                {
                    double costs = this.annualCosts.get(backCounter);
                    double sales = this.annualGross.get(backCounter);
                    double net = sales - costs;
                    lastNyearsSum += net;
                    backCounter--;
                }
            }

            if (!this.isInBusiness() || lastNyearsSum < 0) //if farmer out of business, put out of production
            {
                luToPlant = 4;
                lp.setInProduction(false);
            }
            else //is in business
            {
                if (lp.getCurrLU() == 0 && lp.getYrsInCrp() < PolicyMaker.getCrpLeaseLength())
                {
                    luToPlant = 0;  //if currently in CRP and lease length hasn't been met, stay in CRP
                    lp.setInProduction(false);
                }

                else  //all other cases
                {
                    //figure out costs for each class - all will be in total costs per parcel for the entire time horizon
                    for (int i = 0; i < numClasses; i++)
                    {
                        //plan costs by looking into the future
                        double costs = Market.planCosts(i, lp, this.getTimeHorizon());

                        //modify production costs by weighting HEL and the size of the parcel
                        if(i == 1 || i == 2)
                        {
                            if(acres <= 10) costs *= 1.05;
                            if(hel >= 2)  costs *= 1.10;
                        }
                        expectedCosts.add(costs);
                    }

                    //figure out prices
                    for (int i = 0; i < numClasses; i++)
                    {
                        double expSales = Market.planSales(i, lp, this.getTimeHorizon());
                        expectedSales.add(expSales);
                    }

                    //net is price - costs
                    for (int i = 0; i < expectedCosts.size(); i++)
                    {
                        expectedNet.add(expectedSales.get(i) - expectedCosts.get(i));
                    }


                    //determine highest profit land use
                    int tempLU = 0;
                    for (int i = 0; i < expectedNet.size(); i++)
                    {
                        if (expectedNet.get(i) > expectedNet.get(tempLU))
                        {
                            tempLU = i;
                        }
                    }


                    //if beans or corn, keep in corn->beans->corn rotation
                    if(tempLU == 1 || tempLU == 2)
                    {
                        if (lp.getCurrLU() == 1) tempLU = 2;
                        if (lp.getCurrLU() == 2) tempLU = 1;
                    }



                    luToPlant = tempLU;

                    this.setExpectedAnnualNet(expectedSales.get(luToPlant) - expectedCosts.get(luToPlant));

                    lp.setExpectedSales(expectedSales.get(luToPlant));
                    lp.setLuToPlant(luToPlant);
                }

            }


            if(luToPlant == 0 || luToPlant == 4) lp.setInProduction(false);
            else lp.setInProduction(true);


        }
    }

    public void plantCrops()
    {
        double annCosts = 0;
        for (LandParcel lp : indParcelList)
        {
            double lpSize = lp.getAcres();
            double stochFactor = 1 + (Controller.randomGenerator.nextGaussian() /20);  //costs are never exactly what is expected
            double lpCosts = Market.getLUCost(lp) * stochFactor;
            double tempCosts = lp.getAcres() * lpCosts;
            annCosts += tempCosts;  //tally costs

            lp.updateLU(lp.getLuToPlant());  //update the land use of the parcel
            lp.setCostInField(tempCosts);

            //add any potential switch subsidy to the amount the policymaker has provided
            if(lp.getCurrLU() == 3 && lp.getYrsInSwitch() < 2)
            {
                //if in the first 2 years of switch and if the subsidy isn't 0, then set that it's been subsidized
                if(PolicyMaker.getSwitchGrassSubsidy() > 0) lp.setBeenSubsidized(true);

                double sub = Market.getUnSubSwitchCosts(lp.getYrsInSwitch()) * lpSize * PolicyMaker.switchGrassSubsidy;
                PolicyMaker.updateCurrYrSubsidyTotal(sub);
            }
            //
            //resent counters if not in crp or switch
            if(lp.getCurrLU() != 3) lp.setYrsInSwitch(0);
            if(lp.getCurrLU() != 0) lp.setYrsInCrp(0);
        }
        this.addAnnualCosts(annCosts);
    }

    public void buyInsurance()
    {
        //insurance rates are baked into the costs already - so all parcels are insured
        for(LandParcel lp: indParcelList)
        {
            lp.setInsured(true);
        }
    }

    public void harvestCrops()
    {
        for (LandParcel relevantParcel : indParcelList)
        {
            int lu = relevantParcel.getCurrLU();
            double prod;
            if (lu == 0)
            {
                prod = 1.0; //if crp, then make production 1, so it can be multiplied by $/acre
                relevantParcel.setYrsInCrp(relevantParcel.getYrsInCrp() + 1);  //increment years in crp
            }
            else
            {
                prod = relevantParcel.getCropInField();  //if not crp, use bushels or tons in field

                //then check to see if in continuous switch and increment years
                if (relevantParcel.getLastYrLU() == 3 && relevantParcel.getCurrLU() == 3)
                {
                    relevantParcel.setYrsInSwitch(relevantParcel.getYrsInSwitch() + 1);
                }
            }

            relevantParcel.addYield(relevantParcel.getCropInField());
            relevantParcel.setCropInField(0.0);

            double formerProd = this.getAnnualProduction(lu);
            this.setAnnualProduction(lu, formerProd + prod);

        }
    }

    public void sellCrops()
    {
        double tempGross = 0;
        //sell accumulated amounts of crops
        for (int i = 0; i < annualProduction.length; i++)
        {
            tempGross += (Market.getLUPrice(i) * annualProduction[i]);
            annualProduction[i] = 0;  //zero it back out after selling
        }

        for (LandParcel lp : indParcelList)
        {
            double yield = lp.getYield();
            double price = Market.getLUPrice(lp.getCurrLU());
            double costs = lp.getCostInField();
            lp.updateProfitList((yield * price) - costs);
            lp.setSoldValue(yield * price);

            if((yield * price) * 2 < lp.getExpectedSales() && lp.isInProduction())  //if expected sales are greater than actual by a given amount
            {
                lp.setToBeClaimed(true);
            }
        }

        this.addAnnualGross(tempGross);

    }

    public void collectInsurancePayment()
    {
        double insAmount = 0;

        for(LandParcel lp : indParcelList)
        {
            if(lp.isInsured() && lp.isToBeClaimed())
            {
              //  int tempLU = lp.getCurrLU();
                double expectedSales = lp.getExpectedSales();
                double actualSales = lp.getSoldValue();
                double tempPayment = 0;
                if (actualSales - expectedSales < 0)
                {
                    tempPayment = Market.getInsuranceReimbursementFactor() * (expectedSales - actualSales);
                }
                insAmount += tempPayment;
            }

            lp.setInsured(false);  //reset for next tick
            lp.setToBeClaimed(false);
        }

        double grossLessIns = this.getCurrentYrGross();
        this.setAnnualGross(this.annualGross.size() - 1, insAmount + grossLessIns);
        PolicyMaker.updateCurrYrInsuranceTotal(insAmount);
    }

    public void balanceBooks()
    {
        assert(this.annualCosts.size() == this.annualGross.size());
        assert(this.annualCosts.size() == this.annualInsurance.size());

        //wait a few years for this test
        if(this.annualCosts.size() > 10)
        {
            int listSize = this.annualCosts.size();
            int numYears = 3;

            double lastNyearsSum = 0;

            for(int i = listSize - 1; i > listSize - 1 - numYears; i--)
            {
                double costs = this.annualCosts.get(i);
                double sales = this.annualGross.get(i);
                double net = sales - costs;
                lastNyearsSum += net;
            }

            if (lastNyearsSum/numYears <= 0)
            {
                this.stopFarming();
            }

        }

        //no need to calculate anything else
    }

    public void stopFarming()
    {
        this.setInBusiness(false);
        for(LandParcel lp: indParcelList)
        {
            lp.setInProduction(false);
        }
    }
}
