import java.util.ArrayList;
import java.util.Comparator;

/**
 * Created with IntelliJ IDEA.
 * User: pjbitterman
 * Date: 3/1/13
 * Time: 12:34 PM
 */
public class LandParcel
{
    //class variables
    public static String[] LUClasses = new String[5];  //List of LUClasses available
    //TODO a minor bug in the length of the LU class array

    public static ArrayList<Double> cropTypeFactors = new ArrayList<Double>();

    //instance variables
    private int parcelID;
    private int farmerID; //farmer who manages
    private double xLoc;  //centroid x location
    private double yLoc;  //centroid y location

    private double potentialCornYield;
    private double potentialBeanYield;
    private double potentialWheatYield;        //proxy for switchgrass
    private int yrsInSwitch;
    private int yrsInCrp;
    private double cornSuitabilityRating;

    private double acres;
    private double hel;    //determines whether or not parcels can go into CRP
    private boolean inProduction;

    //USLE variables
    private double k_factor;  //organic matter content - from SSURGO
    private double ls_factor; //estimated from DEM
    private double t_factor;  //tolerable soil loss - from SSURGO
    private double p_factor;

    private int typicalUse;

    private ArrayList<Integer> luList;  //list of land uses, most recent land use always at the last position
    private ArrayList<Integer> tillageList;
    private int luToPlant;
    private boolean isInsured = false;
    private boolean toBeClaimed = false;
    private boolean beenSubsidized;
    private double cropInField;  //post-plant, pre-harvest  <- this is what is perturbed
    private double valueInField; //cost in field
    private double expectedSales;  //expected sales value when planting
    private double soldValue;  //what the parcel's output sold for most recently
    private double cropProduced; //post-harvest
    private ArrayList<Double> yieldList;
    private ArrayList<Double> profitList;


    //empty constructor
    public LandParcel()
    {
        luList = new ArrayList<Integer>();
        luList = new ArrayList<Integer>();
        yieldList = new ArrayList<Double>();
        profitList = new ArrayList<Double>();
        inProduction = true;
        beenSubsidized = false;

        setCropInField(0);
        setYrsInSwitch(0);
        this.setT_factor(0.9);  //assumes basic spring plow
        this.setUsleP(0.75);  //assumes some basic cross-slope row cropping 
    }

    //methods
    public int getParcelID()
    {
        return parcelID;

    }

    public void setParcelID(int parcelID)
    {
        this.parcelID = parcelID;
    }

    public int getFarmerID()
    {
        return farmerID;
    }

    public void setFarmerID(int farmerID)
    {
        this.farmerID = farmerID;
    }

    public double getxLoc()
    {
        return xLoc;
    }

    public void setxLoc(double xLoc)
    {
        this.xLoc = xLoc;
    }

    public double getyLoc()
    {
        return yLoc;
    }

    public void setyLoc(double yLoc)
    {
        this.yLoc = yLoc;
    }

    public boolean isToBeClaimed()
    {
        return toBeClaimed;
    }

    public void setToBeClaimed(boolean toBeClaimed)
    {
        this.toBeClaimed = toBeClaimed;
    }

    public boolean isBeenSubsidized()
    {
        return beenSubsidized;
    }

    public void setBeenSubsidized(boolean beenSubsidized)
    {
        this.beenSubsidized = beenSubsidized;
    }

    public ArrayList<Double> getProfitList()
    {
        return profitList;
    }

    public void setProfitList(ArrayList<Double> profitList)
    {
        this.profitList = profitList;
    }

    public double getYrProfit(int yr)
    {
        return this.profitList.get(yr);
    }

    public double getSoldValue()
    {
        return soldValue;
    }

    public void setSoldValue(double soldValue)
    {
        this.soldValue = soldValue;
    }

    public void updateProfitList(double net)
    {
        this.profitList.add(net);
    }

    public double getPotentialCornYield()
    {
        return potentialCornYield;
    }

    public void setPotentialCornYield(double potentialCornYield)
    {
        this.potentialCornYield = potentialCornYield;

    }

    public double getPotentialBeanYield()
    {
        return potentialBeanYield;
    }

    public void setPotentialBeanYield(double potentialBeanYield)
    {
        this.potentialBeanYield = potentialBeanYield;
    }

    public double getCornSuitabilityRating()
    {
        return cornSuitabilityRating;
    }

    public void setCornSuitabilityRating(double cornSuitabilityRating)
    {
        this.cornSuitabilityRating = cornSuitabilityRating;
    }

    public double getPotentialWheatYield()
    {
        return potentialWheatYield;
    }

    public void setPotentialWheatYield(double potentialWheatYield)
    {
        this.potentialWheatYield = potentialWheatYield;
    }

    public int getYrsInSwitch()
    {
        return yrsInSwitch;
    }

    public void setYrsInSwitch(int yrsInSwitch)
    {
        this.yrsInSwitch = yrsInSwitch;
    }

    public int getYrsInCrp()
    {
        return yrsInCrp;
    }

    public void setYrsInCrp(int yrsInCrp)
    {
        this.yrsInCrp = yrsInCrp;
    }

    public double getUsleK()
    {
        return k_factor;
    }

    public void setUsleK(double k_factor)
    {
        this.k_factor = k_factor;
    }

    public double getUsleLS()
    {
        return ls_factor;
    }

    public void setUsleLS(double ls_factor)
    {
        this.ls_factor = ls_factor;
    }

    public double getT_factor()
    {
        return t_factor;
    }

    public void setT_factor(double t_factor)
    {
        this.t_factor = t_factor;
    }

    public double getUsleP()
    {
        if(this.getCurrLU() == 1 || this.getCurrLU() == 2) return p_factor;
        else return 0.1;
    }

    public void setUsleP(double p_factor)
    {
        this.p_factor = p_factor;
    }

    public static double getUsleCT(int cropType)
    {
        return cropTypeFactors.get(cropType);
    }

    public static void updateUsleCT(int cropType, double ct_factor)
    {
        cropTypeFactors.set(cropType, ct_factor);
    }

    public static void addUsleCT(int cropType, double ct_factor)
    {
        cropTypeFactors.add(cropType, ct_factor);
    }

    public double getYield(int index)
    {
        return this.yieldList.get(index);
    }

    public double getYield()
    {
        return this.yieldList.get(yieldList.size() - 1);
    }

    public void addYield(double yield)
    {
        this.yieldList.add(yield);
    }

    public double calcUsleC()
    {
        double tillFactor;
        int currLU = this.getCurrLU();
        if(currLU == 1 || currLU == 2) //corn or beans
        {
            tillFactor = 0.9; //assumes basic cross-slope - can add complexity later if want
        }
        else //everything else
        {
            tillFactor = 0.25;
        }
        double ct_factor = getUsleCT(currLU);

        double toReturn = tillFactor * ct_factor;
        return toReturn;
    }

    public double getCropInField()
    {
        return cropInField;
    }

    public void setCropInField(double cropInField)
    {
        this.cropInField = cropInField;
    }

    public double getCostInField()
    {
        return valueInField;
    }

    public void setCostInField(double valueInField)
    {
        this.valueInField = valueInField;
    }

    public double getExpectedSales()
    {
        return expectedSales;
    }

    public void setExpectedSales(double expectedSales)
    {
        this.expectedSales = expectedSales;
    }

    public double getCropProduced()
    {
        return cropProduced;
    }

    public void setCropProduced(double cropProduced)
    {
        this.cropProduced = cropProduced;
    }

    public double getAcres()
    {
        return acres;
    }

    public void setAcres(double acres)
    {
        this.acres = acres;
    }

    public double getHel()
    {
        return hel;
    }

    public void setHel(double hel)
    {
        this.hel = hel;
    }

    public int getTypicalUse()
    {
        return typicalUse;
    }

    public void setTypicalUse(int typicalUse)
    {
        this.typicalUse = typicalUse;
    }

    public int getCurrLU()
    {
        int size = this.luList.size();
        return this.luList.get(size-1);  //last index
    }

    public void updateLU(int newLU)
    {
        this.luList.add(newLU);
    }

    public int getLastYrLU()
    {
        int size = this.luList.size();
        return this.luList.get(size-2);  //second to last index
    }

    public int getLuToPlant()
    {
        return luToPlant;
    }

    public void setLuToPlant(int luToPlant)
    {
        this.luToPlant = luToPlant;
    }

    public boolean isInsured()
    {
        return isInsured;
    }

    public void setInsured(boolean insured)
    {
        isInsured = insured;
    }

    public static void setLUClass(int index, String name)
    {
        LUClasses[index] = name;
    }

    public String getLUClass(int index)
    {
        return LUClasses[index];
    }

    public static int getNumberLUClasses()
    {
        return LUClasses.length;
    }

    public boolean isInProduction()
    {
        return inProduction;
    }

    public void setInProduction(boolean inProduction)
    {
        this.inProduction = inProduction;
    }

    public void growCrops()
    {
        double toSet = this.getExpectedYield(this.getCurrLU());

        double rand_factor = 1 + ((Controller.randomGenerator.nextDouble() * 14) -7)/100; //7% variation according to SAGE
        toSet *= rand_factor;
        this.setCropInField(toSet * this.getAcres());
    }

    public double getExpectedYield(int lu)
    {
        double toReturn;

        if(lu == 1)  //corn
        {
            toReturn = this.potentialCornYield;
        }

        else if(lu == 2)  //beans
        {
            toReturn = this.potentialBeanYield;
        }

        else if(lu == 3)  //switchgrass
        {
            int yrs = this.getYrsInSwitch();
            if (yrs == 0)
            {
                toReturn = 0;
            }
            else if (yrs == 1)
            {
                toReturn = 4;
            }
            else if (yrs == 2)
            {
                toReturn = 6;
            }
            else if (yrs == 3)
            {
                toReturn = 7;
            }

            else if (yrs == 4)
            {
                toReturn = 8;
            }

            else  //5 or greater
            {
                toReturn = 10;
            }
            //incrementing the yrsInSwitch variable is taken care of when harvesting
        }

        else  //if crp or fallow
        {
            toReturn = 0;
        }

        return toReturn;
    }

    public void loseSoil()
    {
        double usleA = calcUSLE();
        double usleT = this.getT_factor();
        if (usleT > usleA)
        {
            double lossFactor = 0.05;
            this.setPotentialCornYield((1-lossFactor) * this.getPotentialCornYield());
            this.setPotentialBeanYield((1-lossFactor * this.getPotentialBeanYield()));
            this.setPotentialWheatYield((1-lossFactor * this.getPotentialWheatYield()));
        }
    }

    public double calcUSLE()
    {
        double usleR = Nature.getUsleR();
        double usleK = getUsleK();
        double usleLS = getUsleLS();
        double usleC = calcUsleC();
        double usleP = getUsleP();

        double usleA = usleR * usleK * usleLS * usleC * usleP;
        return usleA;
    }

    public double calcRunoff()
    {
        double precip = Nature.getPrecipitation();
        double temp = Nature.getTemperature();
        double PET = 325 + (21 * temp) + (0.9 * Math.pow(temp,  2));
        double ET = precip * Math.pow(((PET / precip) * (Math.tanh(precip/PET)) * (1 - Math.exp(-1 * PET/precip))), 0.5);
        double Q = precip - ET;

        return Q;  //run-off
    }


    public static void resetLandParcel()
    {   ArrayList<String> LUClasses = new ArrayList<String>();  //List of LUClasses available
        ArrayList<Double> cropTypeFactors = new ArrayList<Double>();
    }


}

