import java.util.ArrayList;

/**
 * Created with IntelliJ IDEA.
 * User: Patrick
 * Date: 3/3/13
 * Time: 6:00 PM
 */
public abstract class Market
{
    //class variables
    public static ArrayList<Double> costList = new ArrayList<Double>();
    public static ArrayList<Double> switchGrassCostList = new ArrayList<Double>();
    public static ArrayList<Double> priceList = new ArrayList<Double>();  //"current" prices
    public static ArrayList<Double> initialPriceList = new ArrayList<Double>();  //save initial prices to reset post-perturbation
    public static double insurancePriceFactor;
    public static double insuranceReimbursementFactor;

    //this method will always return the highest price possible for switchgrass, fine for everything else
    public static double getLUCost(int luClass)
    {
        return costList.get(luClass);
    }

    public static double getLUCost(LandParcel theParcel)  //overloaded to deal with the switchgrass cost inflation problem
    {
        double toReturn;
        if (theParcel.getCurrLU() == 3)
        {
            int grassYear = theParcel.getYrsInSwitch();
            toReturn = getSubSwitchCosts(grassYear); //cost by year
        }
        else //everything else not switchgrass
        {
            toReturn = costList.get(theParcel.getCurrLU());
        }
        return toReturn;
    }

    /**
     * This method returns the costs of planing switchgrass based on how many years the parcel has been in switch
     * @param yr the number of years the parcel has been in switchgrass
     * @return the costs of planting switchgrass
     */
    public static double getSubSwitchCosts(int yr)
    {
        if (yr < switchGrassCostList.size())
        {
            double subsidyFactor = 1;
            if(yr < 2) subsidyFactor = 1 - PolicyMaker.getSwitchGrassSubsidy(); //ex: if subsidy is 20%, then costs are 80%
            return switchGrassCostList.get(yr) * subsidyFactor;
        }
        else return switchGrassCostList.get(switchGrassCostList.size() - 1);
    }

    public static double getUnSubSwitchCosts(int yr)
    {
        if (yr < switchGrassCostList.size())
        {
            return switchGrassCostList.get(yr);
        }
        else return switchGrassCostList.get(switchGrassCostList.size() - 1);
    }

    public static double planCosts(int lu, LandParcel lp, int timeHorizon)
    {
        double luCosts;

        if (lu == 3)  //if switchgrass, find where in the cost sequence it is and then return it - it levels out after year 5
        {
            double costs = 0;
            int switchCostListSize = switchGrassCostList.size();
            double saturatedCost = getSubSwitchCosts(switchCostListSize - 1);
            int startYr = lp.getYrsInSwitch();
            for(int i = startYr; i < startYr + timeHorizon; i++)
            {
                if (i < switchCostListSize)
                {
                    if (i < 2 && !lp.isBeenSubsidized()) costs += getSubSwitchCosts(i);
                    else costs += getUnSubSwitchCosts(i);
                }
                else costs += saturatedCost;
            }

            luCosts = costs;  //this already has the time horizon built in
        }

        else if (lu == 1 || lu == 2)//everything else not switchgrass
        {
            luCosts = 0.5 * (costList.get(1) + costList.get(2)) * timeHorizon;  //avg corn and beans since they're in rotation
        }

        else //everything else (CRP or out of production)
        {
            luCosts = costList.get(lu) * timeHorizon;
        }

        double lpCosts = luCosts * lp.getAcres(); //so that costs are now all in the same units:  $/acre

        return lpCosts;
    }

    public static double planSales(int lu, LandParcel lp, int timeHorizon)
    {
        double luSales;

        //avg corn and beans (in rotation) AND multiply by the acres
        if (lu == 1 || lu == 2)
        {
            double cornValue = lp.getPotentialCornYield() * Market.getLUPrice(1);
            double beanValue = lp.getPotentialBeanYield() * Market.getLUPrice(2);
            luSales = 0.5 * (beanValue + cornValue) * timeHorizon;
        }
        else if(lu == 3)
        {
            double[] prodMatrix = {0.0,2.5,4.0,6.0,8.0,10.0};
            double prod = 0;
            int switchProdListSize = prodMatrix.length;
            double saturatedProd = prodMatrix[prodMatrix.length - 1];
            int startYr = lp.getYrsInSwitch();
            for(int i = startYr; i < startYr + timeHorizon; i++)
            {
                if (i < switchProdListSize)
                {
                    prod += prodMatrix[i];
                }
                else prod += saturatedProd;
            }
            double swPrice = Market.getLUPrice(3);

            luSales = swPrice * prod;  //this already has the time horizon built in
         }
        else  //crp or out of production
        {
            luSales = Market.getLUPrice(lu) * timeHorizon;
        }

        double lpSales = luSales * lp.getAcres();

        return lpSales;
    }

    public static void setSwitchGrassCostList(ArrayList<Double> inList)
    {
        switchGrassCostList = inList;
    }

    public ArrayList<Double> getSwitchGrassList()
    {
        return switchGrassCostList;
    }

    public static void updateLUCost(int luClass, double cost)
    {
        costList.set(luClass, cost);
    }

    public static void addLUCost(int luClass, double cost)
    {
        costList.add(luClass, cost);
    }

    public static double getLUPrice(int luClass)
    {
        return priceList.get(luClass);
    }

    public static void updateLUPrice(int luClass, double price)
    {
        priceList.set(luClass, price);
    }

    public static void addLUPrice(int luClass, double price)
    {
        priceList.add(luClass, price);
    }

    public static double getInitialPrice(int luClass)
    {
        return initialPriceList.get(luClass);
    }

    public static void setInitialPrice(int luClass, double price)
    {
        initialPriceList.add(luClass, price);
    }

    public static double getInsurancePriceFactor()
    {
        return insurancePriceFactor;
    }

    public static void setInsurancePriceFactor(double insurancePriceFactor)
    {
        Market.insurancePriceFactor = insurancePriceFactor;
    }

    public static double getInsuranceReimbursementFactor()
    {
        return insuranceReimbursementFactor;
    }

    public static void setInsuranceReimbursementFactor(double insuranceReimbursementFactor)
    {
        Market.insuranceReimbursementFactor = insuranceReimbursementFactor;
    }

    public static void resetMarket()
    {
        /* class variables */
        costList = new ArrayList<Double>();
        switchGrassCostList = new ArrayList<Double>();
        priceList = new ArrayList<Double>();  //"current" prices
        initialPriceList = new ArrayList<Double>();
        insurancePriceFactor = 0;
        insuranceReimbursementFactor = 0;
    }

}
