import java.util.ArrayList;

/**
 * User: pjbitterman
 * Date: 3/5/13
 * Time: 2:35 PM
 */
public class Scenario
{
    //instance variables
    private int scenarioID;  //scenarioID
    private int scenarioRuns;  //number of times to run the sim
    private int runIterations;  //number of "ticks" or years to run each sim
    private double cStartPrice;  //start price for corn
    private double bStartPrice;  //start price for beans
    private double sStartPrice;  //start price for switchgrass
    private double crpStartPrice;  //start price for crp
    private double insurancePriceFactor;  //how much of the price insurance will reimburse
    private double temperature;  //temperature
    private double rFactor;  //r factor for usle
    private double precipitation;  //used in run-off - different than (but related to) r-factor

    private boolean isDroughtPerturb;
    private int droughtDuration;  //length of drought perturbation
    private double droughtMeanSeverity;  //the proportion by which the average drought reduces yield
    private int perturbStartTick;

    private boolean isPricePerturb;
    private int pricePerturbDuration;
    private double cPricePerturbSeverity;
    private double bPricePerturbSeverity;
    private double sPricePerturbSeverity;

    private int perturbationDelay;
    private int farmerTimeHorizon;
    private int crpLeaseLength;

    double switchGrassSubsidy;
    double insuranceReimbursementFactor;  //the proportion of the price that insurance costs

    private Perturbation thePerturbation;  //holds the perturbation once it's created

    ArrayList<String[]> scenarioData = new ArrayList<String[]>();

    //constructor
    public Scenario(int scenarioID, int scenarioRuns, int runIterations, double cStartPrice, double bStartPrice,
                    double sStartPrice, double crpStartPrice, double insurancePriceFactor, double temperature,
                    double rFactor, double precipitation, boolean isDroughtPerturb, int droughtDuration, double droughtMeanSeverity,
                    boolean isPricePerturb,int pricePerturbDuration, double cPricePerturbSeverity,
                    double bPricePerturbSeverity, double sPricePerturbSeverity, int perturbationDelay,
                    double switchGrassSubsidy, double insuranceReimbursementFactor, int farmerTimeHorizon, int crpLeaseLength)
    {
        this.scenarioID = scenarioID;
        this.scenarioRuns = scenarioRuns;
        this.runIterations = runIterations;
        this.cStartPrice = cStartPrice;
        this.bStartPrice = bStartPrice;
        this.sStartPrice = sStartPrice;
        this.crpStartPrice = crpStartPrice;
        this.insurancePriceFactor = insurancePriceFactor;
        this.temperature = temperature;
        this.rFactor = rFactor;
        this.precipitation = precipitation;
        this.isDroughtPerturb = isDroughtPerturb;
        this.droughtDuration = droughtDuration;
        this.droughtMeanSeverity = droughtMeanSeverity;
        this.isPricePerturb = isPricePerturb;
        this.pricePerturbDuration = pricePerturbDuration;
        this.cPricePerturbSeverity = cPricePerturbSeverity;
        this.bPricePerturbSeverity = bPricePerturbSeverity;
        this.sPricePerturbSeverity = sPricePerturbSeverity;
        this.perturbationDelay = perturbationDelay;
        this.switchGrassSubsidy = switchGrassSubsidy;
        this.insuranceReimbursementFactor = insuranceReimbursementFactor;
        this.farmerTimeHorizon = farmerTimeHorizon;
        this.crpLeaseLength = crpLeaseLength;
    }

    public int getScenarioID()
    {
        return scenarioID;
    }

    public void setScenarioID(int scenarioID)
    {
        this.scenarioID = scenarioID;
    }

    public int getScenarioRuns()
    {
        return scenarioRuns;
    }

    public void setScenarioRuns(int scenarioRuns)
    {
        this.scenarioRuns = scenarioRuns;
    }

    public int getRunIterations()
    {
        return runIterations;
    }

    public void setRunIterations(int runIterations)
    {
        this.runIterations = runIterations;
    }

    public double getcStartPrice()
    {
        return cStartPrice;
    }

    public void setcStartPrice(double cStartPrice)
    {
        this.cStartPrice = cStartPrice;
    }

    public double getbStartPrice()
    {
        return bStartPrice;
    }

    public void setbStartPrice(double bStartPrice)
    {
        this.bStartPrice = bStartPrice;
    }

    public double getsStartPrice()
    {
        return sStartPrice;
    }

    public double getCrpStartPrice()
    {
        return crpStartPrice;
    }

    public void setCrpStartPrice(double crpStartPrice)
    {
        this.crpStartPrice = crpStartPrice;
    }

    public double getInsurancePriceFactor()
    {
        return insurancePriceFactor;
    }

    public void setInsurancePriceFactor(double insurancePriceFactor)
    {
        this.insurancePriceFactor = insurancePriceFactor;
    }

    public double getTemperature()
    {
        return temperature;
    }

    public void setTemperature(double temperature)
    {
        this.temperature = temperature;
    }

    public double getrFactor()
    {
        return rFactor;
    }

    public void setrFactor(double rFactor)
    {
        this.rFactor = rFactor;
    }

    public double getDroughtMeanSeverity()
    {
        return droughtMeanSeverity;
    }

    public void setDroughtMeanSeverity(double droughtMeanSeverity)
    {
        this.droughtMeanSeverity = droughtMeanSeverity;
    }


    public double getbPricePerturbSeverity()
    {
        return bPricePerturbSeverity;
    }

    public void setbPricePerturbSeverity(double bPricePerturbSeverity)
    {
        this.bPricePerturbSeverity = bPricePerturbSeverity;
    }

    public double getsPricePerturbSeverity()
    {
        return sPricePerturbSeverity;
    }

    public void setsPricePerturbSeverity(double sPricePerturbSeverity)
    {
        this.sPricePerturbSeverity = sPricePerturbSeverity;
    }

    public void addToScenarioData(String[] toAdd)
    {
        this.scenarioData.add(toAdd);
    }

    public ArrayList<String[]> getScenarioData()
    {
        return this.scenarioData;
    }

    public String[] getParticularRunData(int index)
    {
        return this.scenarioData.get(index);
    }

    public int getPerturbationDelay()
    {
        return perturbationDelay;
    }

    public void setPerturbationDelay(int perturbationDelay)
    {
        this.perturbationDelay = perturbationDelay;
    }

    public double getSwitchGrassSubsidy()
    {
        return switchGrassSubsidy;
    }

    public void setSwitchGrassSubsidy(double switchGrassSubsidy)
    {
        this.switchGrassSubsidy = switchGrassSubsidy;
    }

    public double getInsuranceReimbursementFactor()
    {
        return insuranceReimbursementFactor;
    }

    public void setInsuranceReimbursementFactor(double insuranceReimbursementFactor)
    {
        this.insuranceReimbursementFactor = insuranceReimbursementFactor;
    }

    public int getDroughtDuration()
    {
        return droughtDuration;
    }

    public void setDroughtDuration(int droughtDuration)
    {
        this.droughtDuration = droughtDuration;
    }

    public int getPricePerturbDuration()
    {
        return pricePerturbDuration;
    }

    public void setPricePerturbDuration(int pricePerturbDuration)
    {
        this.pricePerturbDuration = pricePerturbDuration;
    }

    public double getcPricePerturbSeverity()
    {
        return cPricePerturbSeverity;
    }

    public void setcPricePerturbSeverity(double cPricePerturbSeverity)
    {
        this.cPricePerturbSeverity = cPricePerturbSeverity;
    }


    public boolean isDroughtPerturb()
    {
        return isDroughtPerturb;
    }

    public void setDroughtPerturb(boolean droughtPerturb)
    {
        isDroughtPerturb = droughtPerturb;
    }

    public boolean isPricePerturb()
    {
        return isPricePerturb;
    }

    public void setPricePerturb(boolean pricePerturb)
    {
        this.isPricePerturb = pricePerturb;
    }

    public Perturbation getThePerturbation()
    {
        return thePerturbation;
    }

    public void setThePerturbation(Perturbation thePerturbation)
    {
        this.thePerturbation = thePerturbation;
    }

    public double getPrecipitation()
    {
        return precipitation;
    }

    public void setPrecipitation(double precipitation)
    {
        this.precipitation = precipitation;
    }

    public int getFarmerTimeHorizon()
    {
        return farmerTimeHorizon;
    }

    public void setFarmerTimeHorizon(int farmerTimeHorizon)
    {
        this.farmerTimeHorizon = farmerTimeHorizon;
    }

    public int getCrpLeaseLength()
    {
        return crpLeaseLength;
    }

    public void setCrpLeaseLength(int crpLeaseLength)
    {
        this.crpLeaseLength = crpLeaseLength;
    }


    public int getPerturbStartTick()
    {
        return perturbStartTick;
    }

    public void setPerturbStartTick(int perturbStartTick)
    {
        this.perturbStartTick = perturbStartTick;
    }

    public void endOfRun(int scenarioID, int numberOfTicks, int tempFarmerHorizon, double sumGross, double sumCosts,
                         double sumNet, double runOff, double soilLost, double insPayment, double subPayments,
                         double totCornYield, double totBeanYield, double totSwitchYield, int perturbTick,
                         int farmersInBusiness, int tempLPinProduction, double acresInProduction,
                         int tempLPinSwitch, double tempAcresInSwitch, double startAvgFarmSizeLP, double endAvgFarmSizeLP,
                         double startAvgFarmSizeAcres, double endAvgFarmSizeAcres, double startLandscapeEntropy,
                         double endLandscapeEntropy)
    {

        String[] headArray = {"scID", "number of ticks","farmer horizon", "gross sum",
                "costs sum", "net sum","total run-off", "soil lost", "total ins paid", "total subsidy payments",
                "total corn yield", "total bean yield", "total switch yield","perturb tick", "end farmers in biz",
                "end proportion farmer in biz", "end lp in prod", "end proportion lp in prod", "end acres in prod",
                "end lp in switch", "end acres in switch", "start avg farm size lp","end avg farm size lp",
                "start avg farm size acres", "end avg farm size acres", "start landscape entropy",
                "end landscape entropy"};

        String[] scenarioRunSummary = new String[27];
        scenarioRunSummary[0] =  Integer.toString(scenarioID);
        scenarioRunSummary[1] =  Integer.toString(numberOfTicks);
        scenarioRunSummary[2] =  Integer.toString(tempFarmerHorizon);
        scenarioRunSummary[3] =  Double.toString(sumGross);
        scenarioRunSummary[4] =  Double.toString(sumCosts);
        scenarioRunSummary[5] =  Double.toString(sumNet);
        scenarioRunSummary[6] =  Double.toString(runOff);
        scenarioRunSummary[7] =  Double.toString(soilLost);
        scenarioRunSummary[8] =  Double.toString(insPayment);
        scenarioRunSummary[9] =  Double.toString(subPayments);
        scenarioRunSummary[10] = Double.toString(totCornYield);
        scenarioRunSummary[11] = Double.toString(totBeanYield);
        scenarioRunSummary[12] = Double.toString(totSwitchYield);
        scenarioRunSummary[13] = Integer.toString(perturbTick);
        scenarioRunSummary[14] = Integer.toString(farmersInBusiness);
        scenarioRunSummary[15] = Double.toString((double)farmersInBusiness / (double)Controller.farmerList.size());
        scenarioRunSummary[16] = Integer.toString(tempLPinProduction);
        scenarioRunSummary[17] = Double.toString((double)tempLPinProduction / (double)Controller.parcelList.size());
        scenarioRunSummary[18] = Double.toString(acresInProduction);
        scenarioRunSummary[19] = Integer.toString(tempLPinSwitch);
        scenarioRunSummary[20] = Double.toString(tempAcresInSwitch);
        scenarioRunSummary[21] = Double.toString(startAvgFarmSizeLP);
        scenarioRunSummary[22] = Double.toString(endAvgFarmSizeLP);
        scenarioRunSummary[23] = Double.toString(startAvgFarmSizeAcres);
        scenarioRunSummary[24] = Double.toString(endAvgFarmSizeAcres);
        scenarioRunSummary[25] = Double.toString(startLandscapeEntropy);
        scenarioRunSummary[26] = Double.toString(endLandscapeEntropy);


        if(this.scenarioData.size() == 0)
        {
            addToScenarioData(headArray);
        }

        addToScenarioData(scenarioRunSummary);
    }
}
